from darmilibs.ram.utils.misc import *
from darmilibs.ram.utils.pastebin import *
from darmilibs.ram.utils.sections import *
from darmilibs.ram.utils.tools import *
from darmilibs.ram.utils._adzan import __fetch_adzan
